"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthenticationComponent = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
const express_1 = require("@loopback/express");
const keys_1 = require("./keys");
const middlewares_1 = require("./middlewares");
const providers_1 = require("./providers");
const strategies_1 = require("./strategies");
const keys_2 = require("./strategies/keys");
const secure_client_password_strategy_factory_provider_1 = require("./strategies/passport/passport-client-password/secure-client-password-strategy-factory-provider");
const passport_cognito_oauth2_1 = require("./strategies/passport/passport-cognito-oauth2");
const SAML_1 = require("./strategies/SAML");
let AuthenticationComponent = class AuthenticationComponent {
    constructor(config) {
        var _a, _b, _c;
        this.config = config;
        this.providers = {
            [keys_1.AuthenticationBindings.USER_AUTH_ACTION.key]: providers_1.AuthenticateActionProvider,
            [keys_1.AuthenticationBindings.CLIENT_AUTH_ACTION.key]: providers_1.ClientAuthenticateActionProvider,
            [keys_1.AuthenticationBindings.USER_METADATA.key]: providers_1.AuthMetadataProvider,
            [keys_1.AuthenticationBindings.CLIENT_METADATA.key]: providers_1.ClientAuthMetadataProvider,
            [keys_1.AuthenticationBindings.USER_STRATEGY.key]: strategies_1.AuthStrategyProvider,
            [keys_1.AuthenticationBindings.CLIENT_STRATEGY.key]: strategies_1.ClientAuthStrategyProvider,
            // Strategy function factories
            [keys_2.Strategies.Passport.LOCAL_STRATEGY_FACTORY.key]: strategies_1.LocalPasswordStrategyFactoryProvider,
            [keys_2.Strategies.Passport.OTP_AUTH_STRATEGY_FACTORY.key]: strategies_1.PassportOtpStrategyFactoryProvider,
            [keys_2.Strategies.Passport.CLIENT_PASSWORD_STRATEGY_FACTORY.key]: strategies_1.ClientPasswordStrategyFactoryProvider,
            [keys_2.Strategies.Passport.BEARER_STRATEGY_FACTORY.key]: strategies_1.BearerStrategyFactoryProvider,
            [keys_2.Strategies.Passport.RESOURCE_OWNER_STRATEGY_FACTORY.key]: strategies_1.ResourceOwnerPasswordStrategyFactoryProvider,
            [keys_2.Strategies.Passport.GOOGLE_OAUTH2_STRATEGY_FACTORY.key]: strategies_1.GoogleAuthStrategyFactoryProvider,
            [keys_2.Strategies.Passport.SAML_STRATEGY_FACTORY.key]: SAML_1.SamlStrategyFactoryProvider,
            [keys_2.Strategies.Passport.INSTAGRAM_OAUTH2_STRATEGY_FACTORY.key]: strategies_1.InstagramAuthStrategyFactoryProvider,
            [keys_2.Strategies.Passport.FACEBOOK_OAUTH2_STRATEGY_FACTORY.key]: strategies_1.FacebookAuthStrategyFactoryProvider,
            [keys_2.Strategies.Passport.APPLE_OAUTH2_STRATEGY_FACTORY.key]: strategies_1.AppleAuthStrategyFactoryProvider,
            [keys_2.Strategies.Passport.AZURE_AD_STRATEGY_FACTORY.key]: strategies_1.AzureADAuthStrategyFactoryProvider,
            [keys_2.Strategies.Passport.KEYCLOAK_STRATEGY_FACTORY.key]: strategies_1.KeycloakStrategyFactoryProvider,
            [keys_2.Strategies.Passport.COGNITO_OAUTH2_STRATEGY_FACTORY.key]: passport_cognito_oauth2_1.CognitoStrategyFactoryProvider,
            // Verifier functions
            [keys_2.Strategies.Passport.OAUTH2_CLIENT_PASSWORD_VERIFIER.key]: strategies_1.ClientPasswordVerifyProvider,
            [keys_2.Strategies.Passport.LOCAL_PASSWORD_VERIFIER.key]: strategies_1.LocalPasswordVerifyProvider,
            [keys_2.Strategies.Passport.OTP_VERIFIER.key]: strategies_1.OtpVerifyProvider,
            [keys_2.Strategies.Passport.BEARER_TOKEN_VERIFIER.key]: strategies_1.BearerTokenVerifyProvider,
            [keys_2.Strategies.Passport.RESOURCE_OWNER_PASSWORD_VERIFIER.key]: strategies_1.ResourceOwnerVerifyProvider,
            [keys_2.Strategies.Passport.GOOGLE_OAUTH2_VERIFIER.key]: strategies_1.GoogleAuthVerifyProvider,
            [keys_2.Strategies.Passport.SAML_VERIFIER.key]: SAML_1.SamlVerifyProvider,
            [keys_2.Strategies.Passport.INSTAGRAM_OAUTH2_VERIFIER.key]: strategies_1.InstagramAuthVerifyProvider,
            [keys_2.Strategies.Passport.FACEBOOK_OAUTH2_VERIFIER.key]: strategies_1.FacebookAuthVerifyProvider,
            [keys_2.Strategies.Passport.COGNITO_OAUTH2_VERIFIER.key]: passport_cognito_oauth2_1.CognitoAuthVerifyProvider,
            [keys_2.Strategies.Passport.APPLE_OAUTH2_VERIFIER.key]: strategies_1.AppleAuthVerifyProvider,
            [keys_2.Strategies.Passport.AZURE_AD_VERIFIER.key]: strategies_1.AzureADAuthVerifyProvider,
            [keys_2.Strategies.Passport.KEYCLOAK_VERIFIER.key]: strategies_1.KeycloakVerifyProvider,
        };
        if ((_a = this.config) === null || _a === void 0 ? void 0 : _a.secureClient) {
            this.providers = {
                ...this.providers,
                [keys_2.Strategies.Passport.CLIENT_PASSWORD_STRATEGY_FACTORY.key]: secure_client_password_strategy_factory_provider_1.SecureClientPasswordStrategyFactoryProvider,
            };
        }
        this.bindings = [];
        if ((_b = this.config) === null || _b === void 0 ? void 0 : _b.useClientAuthenticationMiddleware) {
            this.bindings.push((0, express_1.createMiddlewareBinding)(middlewares_1.ClientAuthenticationMiddlewareProvider));
        }
        if ((_c = this.config) === null || _c === void 0 ? void 0 : _c.useUserAuthenticationMiddleware) {
            this.bindings.push((0, express_1.createMiddlewareBinding)(middlewares_1.UserAuthenticationMiddlewareProvider));
        }
    }
};
AuthenticationComponent = tslib_1.__decorate([
    tslib_1.__param(0, (0, core_1.inject)(keys_1.AuthenticationBindings.CONFIG, { optional: true })),
    tslib_1.__metadata("design:paramtypes", [Object])
], AuthenticationComponent);
exports.AuthenticationComponent = AuthenticationComponent;
//# sourceMappingURL=component.js.map