"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CognitoStrategyFactoryProvider = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
const rest_1 = require("@loopback/rest");
const https_proxy_agent_1 = require("https-proxy-agent");
const keys_1 = require("../../keys");
const CognitoStrategy = require('passport-cognito-oauth2');
let CognitoStrategyFactoryProvider = class CognitoStrategyFactoryProvider {
    constructor(verifierCognito) {
        this.verifierCognito = verifierCognito;
    }
    value() {
        return (options, verifier) => this.getCognitoAuthStrategyVerifier(options, verifier);
    }
    getCognitoAuthStrategyVerifier(options, verifierPassed) {
        const verifyFn = verifierPassed !== null && verifierPassed !== void 0 ? verifierPassed : this.verifierCognito;
        let strategy;
        if (options && options.passReqToCallback === true) {
            strategy = new CognitoStrategy(options, async (req, accessToken, refreshToken, profile, cb) => {
                try {
                    const user = await verifyFn(accessToken, refreshToken, profile, cb, req);
                    if (!user) {
                        throw new rest_1.HttpErrors.Unauthorized("Invalid Credentials" /* AuthErrorKeys.InvalidCredentials */);
                    }
                    cb(undefined, user);
                }
                catch (err) {
                    cb(err);
                }
            });
        }
        else {
            strategy = new CognitoStrategy(options, async (accessToken, refreshToken, profile, cb) => {
                try {
                    const user = await verifyFn(accessToken, refreshToken, profile, cb);
                    if (!user) {
                        throw new rest_1.HttpErrors.Unauthorized("Invalid Credentials" /* AuthErrorKeys.InvalidCredentials */);
                    }
                    cb(undefined, user);
                }
                catch (err) {
                    cb(err);
                }
            });
        }
        this._setupProxy(strategy);
        return strategy;
    }
    _setupProxy(strategy) {
        // Setup proxy if any
        let httpsProxyAgent;
        if (process.env['https_proxy']) {
            httpsProxyAgent = new https_proxy_agent_1.HttpsProxyAgent(process.env['https_proxy']);
            strategy._oauth2.setAgent(httpsProxyAgent);
        }
        else if (process.env['HTTPS_PROXY']) {
            httpsProxyAgent = new https_proxy_agent_1.HttpsProxyAgent(process.env['HTTPS_PROXY']);
            strategy._oauth2.setAgent(httpsProxyAgent);
        }
    }
};
CognitoStrategyFactoryProvider = tslib_1.__decorate([
    tslib_1.__param(0, (0, core_1.inject)(keys_1.Strategies.Passport.COGNITO_OAUTH2_VERIFIER)),
    tslib_1.__metadata("design:paramtypes", [Function])
], CognitoStrategyFactoryProvider);
exports.CognitoStrategyFactoryProvider = CognitoStrategyFactoryProvider;
//# sourceMappingURL=cognito-auth-strategy-factory-provider.js.map